//
//  DeviceV2TableViewController.m
//  DXRiskPocDemo
//
//  Created by admin on 2024/7/9.
//  Copyright © 2024 weaiken. All rights reserved.
//

#import "DeviceV2TableViewController.h"
#import <DXRisk/DXRiskManager.h>
#import <AppTrackingTransparency/AppTrackingTransparency.h>
#import <AdSupport/ASIdentifierManager.h>
#import "MD5Utils.h"
#import "UnityUtil.h"
#import <CoreLocation/CoreLocation.h>




@interface DeviceV2TableViewController ()
@property (weak, nonatomic) IBOutlet UITextField *appIdTf;
@property (weak, nonatomic) IBOutlet UITextField *appSecTf;
@property (weak, nonatomic) IBOutlet UITextField *urlTf;
@property (weak, nonatomic) IBOutlet UILabel *requestLabel;
@property (weak, nonatomic) IBOutlet UITextView *textView;
@property (weak, nonatomic) IBOutlet UISwitch *tokenSwitch;

@property (weak, nonatomic) IBOutlet UILabel *num1Label;
@property (weak, nonatomic) IBOutlet UILabel *num2Label;
@property (weak, nonatomic) IBOutlet UISwitch *switch1;
@property (weak, nonatomic) IBOutlet UISwitch *switch2;
@end

@implementation DeviceV2TableViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    self.clearsSelectionOnViewWillAppear = NO;
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(qrNotifation:) name:@"qrNoti" object:nil];
    
    //态势感知注册
    [[NSNotificationCenter defaultCenter] addObserverForName:@"DXRISKSITUATION" object:self queue:[NSOperationQueue currentQueue] usingBlock:^(NSNotification * _Nonnull notification) {
        NSLog(@"态势感知返回值-%@",notification.object);
    }];
    
    UITapGestureRecognizer * tap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(endEdit:)];
    [self.view addGestureRecognizer:tap];
    
    if ([CLLocationManager authorizationStatus] == kCLAuthorizationStatusNotDetermined) {
        NSLog(@"请求定位授权");
        CLLocationManager * locationManager = [[CLLocationManager alloc] init];
        [locationManager requestWhenInUseAuthorization];
    }
    
    if (@available(iOS 14, *)) {
        [ATTrackingManager requestTrackingAuthorizationWithCompletionHandler:^(ATTrackingManagerAuthorizationStatus status) {
            if (status == ATTrackingManagerAuthorizationStatusAuthorized) {
                NSString *idfa = [[ASIdentifierManager sharedManager].advertisingIdentifier UUIDString];
                NSLog(@"idfa: %@",idfa);
            } else {
                NSLog(@"请在设置-隐私-跟踪中允许App请求跟踪");
            }
        }];
    } else {
        if ([[ASIdentifierManager sharedManager] isAdvertisingTrackingEnabled]) {
            NSString *idfa = [[ASIdentifierManager sharedManager].advertisingIdentifier UUIDString];
            NSLog(@"idfa: %@",idfa);
        } else {
            NSLog(@"请在设置-隐私-广告中打开广告跟踪功能");
        }
    }
    
    NSUserDefaults *ud = [NSUserDefaults standardUserDefaults];
    if ([ud objectForKey:@"urlKey"]) {
        self.urlTf.text = [ud objectForKey:@"urlKey"];
    }
    if ([ud objectForKey:@"appIdKey"]) {
        self.appIdTf.text = [ud objectForKey:@"appIdKey"];
    }
    if ([ud objectForKey:@"appSecKey"]) {
        self.appSecTf.text = [ud objectForKey:@"appSecKey"];
    }
}

- (void)dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)endEdit:(id)sender {
    [self.view endEditing:YES];
}

- (void)qrNotifation:(NSNotification *)qrNoti {
    NSLog(@"qr: %@",qrNoti.object);
    NSString *qrString = qrNoti.object;
    
    NSDictionary * dic = [self dictionaryWithJsonString:qrString];
    if (!dic) {
        [self showAlert:[NSString stringWithFormat:@"非法JSON\n%@",qrString]];
    }
    NSString * keyURL = dic[@"keyURL"];
    NSString * appId = dic[@"appId"];
    NSString * appSec = dic[@"secret"];
    
    self.urlTf.text = keyURL;
    self.appIdTf.text = appId;
    self.appSecTf.text = appSec;
}

-(IBAction)actions:(id)sender {
    UIAlertController * alert = [UIAlertController alertControllerWithTitle:@"顶象" message:@"请选择" preferredStyle:UIAlertControllerStyleActionSheet];
    
    UIAlertAction *action1 = [UIAlertAction actionWithTitle:@"检测Url，超时时间5秒" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        [self checkURL:self.urlTf.text withTimeout:5];
    }];
//    UIAlertAction *action2 = [UIAlertAction actionWithTitle:@"其他..." style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
//        [self showAlert:@"暂无"];
//    }];
    UIAlertAction *cancel = [UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleDestructive handler:^(UIAlertAction * _Nonnull action) {
        [alert dismissViewControllerAnimated:YES completion:nil];
    }];
    
    [alert addAction:action1];
    [alert addAction:cancel];
    [self presentViewController:alert animated:YES completion:nil];
}

#pragma mark - 事件

//态势感知主动掉起
- (IBAction)upSituation:(UIButton *)sender {
    sender.enabled = NO;
    dispatch_async(dispatch_get_global_queue(0, 0), ^{
        
        __block BOOL isRun = YES;
        dispatch_semaphore_t semaphore = dispatch_semaphore_create(0);
        while (isRun) {
            [DXRiskManager updateSituation];
            __block int value = 2;
            dispatch_async(dispatch_get_main_queue(), ^{
                value = [self.num2Label.text intValue];
                isRun = self.switch2.isOn;
                dispatch_semaphore_signal(semaphore);
            });
            dispatch_semaphore_wait(semaphore, DISPATCH_TIME_FOREVER);
            if (isRun) {
                NSLog(@"态势等待:%d",value);
                sleep(value);
            }
        }
        dispatch_async(dispatch_get_main_queue(), ^{
            sender.enabled = YES;
        });
    });
}

- (IBAction)getToken:(UIButton *)sender {
    self.textView.text = @"";
    self.textView.scrollsToTop=YES;
    self.requestLabel.text = @"";
    
    NSString *url = self.urlTf.text;
    NSString *appId =self.appIdTf.text;
    NSString *appSec = self.appSecTf.text;
    
    if (url.length == 0 || appId.length == 0 || appSec.length == 0) {
        [self showAlert:@"URL、AppId、AppSec 不能为空"];
        return ;
    }
    
    //SDK初始化,仅做展示,具体调用位置见开发文档
    BOOL isSuccess = [DXRiskManager setup];
    NSLog(@"初始化是否成功:%@",isSuccess?@"是":@"否");
//    NSString *mTokenUrl = url;
    NSString *mTokenUrl = [NSString stringWithFormat:@"%@%@", url, @"/udid/m1"];
    NSString *mEngineUrl = [NSString stringWithFormat:@"%@%@", url, @"/udid/api/getDeviceInfo"];
    NSString *sTokenUrl = [NSString stringWithFormat:@"%@%@", url, @"/udid/sa-m"];
    
    //SDK上报配置项
    NSMutableDictionary *Params = [NSMutableDictionary new];
    //此项为预留项,可填可不填
    Params[DXRiskManagerKeyUserId] = @"123456";
    //必填项目
    Params[DXRiskManagerKeyURL] = mTokenUrl;
    Params[DXRiskManagerSituationUrl] = sTokenUrl;
    //缓存 token 开关
    Params[@"PRIVATE_CLEAR_TOKEN"] = @"false";
    //SDK场景上报
    [DXRiskManager beginScene:@"login"];
    NSString *requestInfo = [NSString stringWithFormat:@""];
    requestInfo = [requestInfo stringByAppendingFormat:@"%@%@%@" ,@"appId:" , appId , @"\r\n"];
    requestInfo = [requestInfo stringByAppendingFormat:@"%@%@%@" ,@"appSec:" , appSec , @"\r\n"];
    requestInfo = [requestInfo stringByAppendingFormat:@"%@%@%@" ,@"risk url:" , mTokenUrl , @"\r\n"];
    requestInfo = [requestInfo stringByAppendingFormat:@"%@%@%@" ,@"info url:" , mEngineUrl , @"\r\n\n"];

    self.requestLabel.text = requestInfo;
    
    double nowtime = [self currentTime];
    sender.enabled = NO;
    // 异步耗时操作
    dispatch_async(dispatch_get_global_queue(0, 0), ^{
        /**
         *
         * SDK token 获取
         * appid
         * 配置项
         * 业务 sid (如没有置为空)
         */
        
        __block BOOL isRun = YES;
        int loopCount = 0;
        while (isRun) {
            NSLog(@"第(%d)请求", loopCount++);
            NSString *token = [DXRiskManager getToken:appId extendsParams: Params WithSid:@""];
            
            NSString *response = [NSString stringWithFormat:@"%@%@" , @"----请求结果----" , @"\r\n" ];
            response = [response stringByAppendingFormat:@"%@%@%@" ,@"token:" , token, @"\r\n"];

            //此项为 Demo 专用接口,用来展示设备指纹以及风险等采集
            NSString *reponseDevice = [self getDeviceInfo:token andAppId:appId andAppSecret:appSec andRequestUrl:mEngineUrl];

            response = [response stringByAppendingFormat:@"%@%@"  , reponseDevice , @"\r\n\n"];

            response = [response stringByAppendingFormat:@"%@%@"   , @"----性能消耗信息----" ,@"\r\n"];

            NSString *memuse = [NSString stringWithFormat:@"%.1fKB\r",[UnityUtil memoryUsage]];
            NSString *cpuuse = [NSString stringWithFormat:@"%.2f%%\r",[UnityUtil cpuUsage]];
            NSString *memshow = [NSString stringWithFormat:@"%@%@\n",@"内存使用:",memuse];
            NSString *cpushow = [NSString stringWithFormat:@"%@%@\n",@"CPU使用:",cpuuse];
            NSString *showtime = [NSString stringWithFormat:@"%@%.0f ms\n",@"采集耗时:",[self currentTime]-nowtime];

            NSString *showText = [NSString stringWithFormat:@"%@%@%@",memshow,cpushow,showtime];
            response = [response stringByAppendingFormat:@"%@"   , showText];
            
            
            __block int waitFor = 2;
            dispatch_semaphore_t semaphore = dispatch_semaphore_create(0);
            dispatch_async(dispatch_get_main_queue(), ^{
                self.textView.text = [NSString stringWithFormat:@"%@" , response ];
                isRun = self.switch1.isOn;
                waitFor = [self.num1Label.text intValue];
                dispatch_semaphore_signal(semaphore);
            });
            dispatch_semaphore_wait(semaphore, DISPATCH_TIME_FOREVER);
            if (isRun) {
                NSLog(@"等待: %d",waitFor);
                sleep(waitFor);
            }
        }
        
        dispatch_async(dispatch_get_main_queue(), ^{
            sender.enabled = YES;
        });
        
        NSLog(@"执行结束");
        [[NSUserDefaults standardUserDefaults] setObject:url forKey:@"urlKey"];
        [[NSUserDefaults standardUserDefaults] setObject:appId forKey:@"appIdKey"];
        [[NSUserDefaults standardUserDefaults] setObject:appSec forKey:@"appSecKey"];
    });
    

}

#pragma mark - other

- (NSDictionary *)dictionaryWithJsonString:(NSString *)jsonString
{
    if (jsonString == nil) {
        return nil;
    }

    NSData *jsonData = [jsonString dataUsingEncoding:NSUTF8StringEncoding];
    NSError *err;
    NSDictionary *dic = [NSJSONSerialization JSONObjectWithData:jsonData
                                                        options:NSJSONReadingMutableContainers
                                                          error:&err];
    if(err)
    {
        return nil;
    }
    return dic;
}


//获取当前时间戳
-(double)currentTime{
    NSDate* date = [NSDate dateWithTimeIntervalSinceNow:0];
    NSTimeInterval time=[date timeIntervalSince1970]*1000;
    return time;
}


- (NSString*) getDeviceInfo:(NSString*) token andAppId:(NSString*) AppID andAppSecret:(NSString*) AppSecret andRequestUrl:(NSString*) requestUrl {
    NSString* mdStr = [MD5Utils getmd5WithString:[NSString stringWithFormat:@"%@%@%@", AppSecret, token , AppSecret]];
    NSString* url = [requestUrl stringByAppendingString:@"?appId="];
    url = [url stringByAppendingString:AppID];
    url = [url stringByAppendingString:@"&token="];
    url = [url stringByAppendingString: [NSString stringWithString:[token stringByReplacingPercentEscapesUsingEncoding:NSUTF8StringEncoding]]];
    url = [url stringByAppendingString:@"&sign="];
    url = [url stringByAppendingString:mdStr];

    return [self httpGet:url];
}


- (NSString*) httpGet:(NSString*) url{
    NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:[NSURL URLWithString:url]];
    request.HTTPMethod = @"get";
    [request setValue:@"*/*" forHTTPHeaderField:@"Accept"];
    [request setValue:@"STEE-SDK" forHTTPHeaderField:@"User-Agent"];
    [request setValue:@"keep-Alive" forHTTPHeaderField:@"connection"];

    NSError *error = nil;
    NSURLResponse *response = nil;
    //发送同步请求(sendSynchronousRequest)
    NSData *data = [NSURLConnection sendSynchronousRequest:request returningResponse:&response error:&error];

    if (!error) {
        //打印的服务端返回的信息以及错误信息
//        NSLog(@"%@",[[NSString alloc]initWithData:data encoding:NSUTF8StringEncoding]);
//        NSLog(@"request-reponse--%@",error);
        return [[NSString alloc]initWithData:data encoding:NSUTF8StringEncoding];

    }else {
//        NSLog(@"request-reponse--%@",error);
        return [[NSString alloc]initWithData:data encoding:NSUTF8StringEncoding];
    }
}

- (void)checkURL:(NSString *)urlString withTimeout:(NSTimeInterval)timeout {
    NSURL *url = [NSURL URLWithString:urlString];
    if (!url) {
        NSLog(@"Invalid URL");
        [self showAlert:@"Invalid URL"];
        return;
    }

    NSURLSessionConfiguration *configuration = [NSURLSessionConfiguration defaultSessionConfiguration];
    configuration.timeoutIntervalForRequest = timeout; // 设置请求超时时间
    configuration.timeoutIntervalForResource = timeout; // 设置资源超时时间

    NSURLSession *session = [NSURLSession sessionWithConfiguration:configuration];
    NSURLSessionDataTask *task = [session dataTaskWithURL:url
                                        completionHandler:^(NSData *data, NSURLResponse *response, NSError *error) {
        if (error) {
            NSLog(@"Error: %@", error.localizedDescription);
            [self showAlert:error.description];
        } else {
            NSHTTPURLResponse *httpResponse = (NSHTTPURLResponse *)response;
            if (httpResponse.statusCode == 200) {
                NSLog(@"URL is available");
                [self showAlert:@"URL is available"];
            } else {
                NSLog(@"URL is not available, Status Code: %ld", (long)httpResponse.statusCode);
                [self showAlert:[NSString stringWithFormat:@"status code: %ld",httpResponse.statusCode]];
            }
        }
    }];
    [task resume];
}



- (void)showAlert:(NSString *)msg {
    if (!msg) {
        return;
    }
    if ([NSThread isMainThread]) {
        UIAlertController *alert = [UIAlertController alertControllerWithTitle:nil message:msg preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *action = [UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDestructive handler:^(UIAlertAction * _Nonnull action) {
            [alert dismissViewControllerAnimated:YES completion:nil];
        }];
        [alert addAction:action];
        [self presentViewController:alert animated:YES completion:nil];
    }else {
        dispatch_async(dispatch_get_main_queue(), ^{
            UIAlertController *alert = [UIAlertController alertControllerWithTitle:nil message:msg preferredStyle:UIAlertControllerStyleAlert];
            UIAlertAction *action = [UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDestructive handler:^(UIAlertAction * _Nonnull action) {
                [alert dismissViewControllerAnimated:YES completion:nil];
            }];
            [alert addAction:action];
            [self presentViewController:alert animated:YES completion:nil];
        });
    }
}


#pragma mark -

- (IBAction)senderstepChange1:(UIStepper *)stepper {
    self.num1Label.text = [NSString stringWithFormat:@"%.0f",stepper.value];
}

- (IBAction)stepChange2:(UIStepper *)stepper {
    self.num2Label.text = [NSString stringWithFormat:@"%.0f",stepper.value];
}
@end
