//
//  QRCodeScannerViewController.m
//  DXRiskPocDemo
//
//  Created by admin on 2024/7/9.
//  Copyright © 2024 weaiken. All rights reserved.
//

#import "QRCodeScannerViewController.h"

@interface QRCodeScannerViewController () <AVCaptureMetadataOutputObjectsDelegate>

@property (strong, nonatomic) AVCaptureSession *session;
@property (strong, nonatomic) AVCaptureVideoPreviewLayer *previewLayer;

@end

@implementation QRCodeScannerViewController

- (void)viewDidLoad {
    [super viewDidLoad];
   
    // 初始化捕捉会话
    self.session = [[AVCaptureSession alloc] init];
    
    // 获取视频捕捉设备（默认是后置摄像头）
    AVCaptureDevice *videoCaptureDevice = [AVCaptureDevice defaultDeviceWithMediaType:AVMediaTypeVideo];
    NSError *error;
    AVCaptureDeviceInput *videoInput = [AVCaptureDeviceInput deviceInputWithDevice:videoCaptureDevice error:&error];
    
    if (videoInput) {
        [self.session addInput:videoInput];
    } else {
        NSLog(@"Error: %@", error);
        return;
    }
    
    // 初始化元数据输出
    AVCaptureMetadataOutput *metadataOutput = [[AVCaptureMetadataOutput alloc] init];
    [self.session addOutput:metadataOutput];
    
    [metadataOutput setMetadataObjectsDelegate:self queue:dispatch_get_main_queue()];
    [metadataOutput setMetadataObjectTypes:@[AVMetadataObjectTypeQRCode]];
    
    // 初始化预览图层
    self.previewLayer = [AVCaptureVideoPreviewLayer layerWithSession:self.session];
    self.previewLayer.frame = self.view.layer.bounds;
    self.previewLayer.videoGravity = AVLayerVideoGravityResizeAspectFill;
    [self.view.layer addSublayer:self.previewLayer];
    
    // 开始会话
    [self.session startRunning];
}

- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear: animated];
    
    // 获取默认的摄像头设备（后置摄像头）
    AVCaptureDevice *cameraDevice = [AVCaptureDevice defaultDeviceWithMediaType:AVMediaTypeVideo];
    // 判断设备是否存在
    if (!cameraDevice) {
        NSLog(@"摄像头不可用");
        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(1 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
            [self.navigationController popToRootViewControllerAnimated:YES];
        });
        return ;
    }
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    // 停止会话
    [self.session stopRunning];
}

// 处理捕捉到的元数据对象
- (void)captureOutput:(AVCaptureOutput *)captureOutput didOutputMetadataObjects:(NSArray<AVMetadataObject *> *)metadataObjects fromConnection:(AVCaptureConnection *)connection {
    for (AVMetadataObject *metadata in metadataObjects) {
        if ([metadata.type isEqualToString:AVMetadataObjectTypeQRCode]) {
            AVMetadataMachineReadableCodeObject *qrCodeObject = (AVMetadataMachineReadableCodeObject *)metadata;
            NSString *qrCodeString = qrCodeObject.stringValue;
            
            // 停止会话
            [self.session stopRunning];
            /*
            // 处理二维码数据
            UIAlertController *alert = [UIAlertController alertControllerWithTitle:@"QR Code" message:qrCodeString preferredStyle:UIAlertControllerStyleAlert];
            [alert addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:^(UIAlertAction *action) {
                [self.session startRunning]; // 重新开始会话
            }]];
            [self presentViewController:alert animated:YES completion:nil];
             */
            [[NSNotificationCenter defaultCenter] postNotificationName:@"qrNoti" object:qrCodeString];
            dispatch_async(dispatch_get_main_queue(), ^{
                [self.navigationController popViewControllerAnimated:YES];
            });
            break;
        }
    }
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
